
local lg = love.graphics
local lm = love.math

local bgSky,bgLand

local skyX,skyY,landX,landY

local speed = 200
local alpha = 0
local angspeed = 5/4*math.pi

local playerImg = lg.newImage("player.png")

local function computeLand()
	local abs=math.abs
	local id = love.image.newImageData(SW,SH)

	local maxr=1
	local minr=0.5
	local yrad=2*maxr - minr
	
	local ox= 200*math.random() - 10
	local oy= 20*math.random() - 10
	local oz= 20*math.random() - 10

	for x=0,SW-1 do
		local alpha = 2*x/SW*math.pi
		for y=0,SH-1 do
			local beta = 2 *y/SH*math.pi
			local yy= oy+yrad*math.sin(beta)
			local ur = maxr - minr*math.cos(beta)
			local xx = ox+ur*math.cos(alpha)
			local zz = oz+ur*math.sin(alpha)
			local u1 = lm.noise(xx,yy,zz)
			local u2 = lm.noise(2*xx,2*yy,2*zz)
			local u3 = lm.noise(4*xx,4*yy,4*zz)
			local u4 = lm.noise(8*xx,8*yy,8*zz)
			local u = u1 + u2/2 + u3/4 + u4/8
			u=u/1.875
			u=u*u
			local cr,cb,cg
			if u < 0.5 then
				cb = u + 0.5 
				cr=0
				cg=0
			else
				cb = 0
				cg = u
				cr = 2*(u -0.5)
			end
			id:setPixel(x,y,196*cr,196*cg,196*cb)
		end
	end
	
	local sx = math.random(100,300)
	local sy = math.random(100,300)
	lg.setCanvas(bgLand)
	local img = lg.newImage(id)
	lg.draw(img,-sx,-sy)
	lg.draw(img,SW-sx,-sy)
	lg.draw(img,-sx,SH-sy)
	lg.draw(img,SW-sx,SH-sy)
	lg.setCanvas()
end


local function computeSky()
	local abs=math.abs
	local id = love.image.newImageData(SW,SH)
	local cc={86,96,80}
	local maxr=1
	--local minr=0.8
	local minr=0.5
	--local yrad=maxr/minr
	local yrad=2*maxr - minr
	
	local ox= 50*math.random() - 25
	local oy= 50*math.random() - 25
	local oz= 50*math.random() - 25

	for x=0,SW-1 do
		local alpha = 2*x/SW*math.pi
		for y=0,SH-1 do
			local beta = 2 *y/SH*math.pi
			local yy= oy+yrad*math.sin(beta)
			local ur = maxr - minr*math.cos(beta)
			local xx = ox+ur*math.cos(alpha)
			local zz = oz+ur*math.sin(alpha)
			local u1 = lm.noise(xx,yy,zz)
			local u2 = lm.noise(2*xx,2*yy,2*zz)
			local u3 = lm.noise(4*xx,4*yy,4*zz)
			local u4 = lm.noise(8*xx,8*yy,8*zz)
			local u = u1 + u2/2 + u3/4 + u4/8
			--u=4*u-math.floor(4*u)
			u=u/1.875
			id:setPixel(x,y,cc[1],cc[2],cc[3],u*255)
		end
	end
	
	local sx = math.random(100,300)
	local sy = math.random(100,300)

	lg.setCanvas(bgSky)
	bgSky:clear()
	local img = lg.newImage(id)
	lg.draw(img,-sx,-sy)
	lg.draw(img,SW-sx,-sy)
	lg.draw(img,-sx,SH-sy)
	lg.draw(img,SW-sx,SH-sy)
	lg.setCanvas()
end

function update_load(dt)
	if loadState == "start" then
		loadState="land"
	elseif loadState == "land" then
		computeLand()
		loadState="sky"
	elseif loadState == "sky" then
		computeSky()
		loadState="end"
	else
		love.update = update_dyn
	end
end

function update_dyn(dt)
	if love.keyboard.isDown("left") then
		alpha = alpha - angspeed * dt
	elseif love.keyboard.isDown("right") then
		alpha = alpha + angspeed * dt
	end

    local disp = speed * dt
	local dx = -disp * math.sin(alpha)
	local dy = disp * math.cos(alpha)
	
	skyX  = skyX + dx
	skyY  = skyY + dy
	landX = landX + dx/5
	landY = landY + dy/5
	
	if landX > SW then landX = landX - SW end
	if landY > SH then landY = landY - SH end
	if landX < 0 then landX = landX+SW end
	if landY < 0 then landY = landY+SH end
	
	if skyX > SW then skyX = skyX - SW end
	if skyY > SH then skyY = skyY - SH end
	if skyX < 0 then skyX = skyX+SW end
	if skyY < 0 then skyY = skyY+SH end
	
end

function love.load()
	math.randomseed(os.time())
	SW,SH = lg.getWidth(), lg.getHeight()

	bgSky = lg.newCanvas(SW,SH)
	bgLand = lg.newCanvas(SW,SH)

	skyX=0
	skyY=0
	landX=0
	landY=0
	reset()
end

function reset()
	loadState="start"
	love.update = update_load
end

function love.draw()
	lg.draw(bgLand,landX,landY)
	lg.draw(bgLand,landX-SW,landY-SH)
	lg.draw(bgLand,landX-SW,landY)
	lg.draw(bgLand,landX,landY-SH)
	
	lg.draw(bgSky,skyX,skyY)
	lg.draw(bgSky,skyX-SW,skyY-SH)
	lg.draw(bgSky,skyX-SW,skyY)
	lg.draw(bgSky,skyX,skyY-SH)
	
	lg.draw(playerImg,SW/2,SH/2,alpha,1,1,32,32)
end

function love.keypressed(key)
	if key==" " or key =="return" then
		reset()
	end
end
